@@ -22,7 +22,7 @@
 
 (defpackage #:postgresql-socket3
   (:use #:cl md5 #:cl-postgres)
-  (:shadow #:postgresql-warning)
+  (:shadow #:postgresql-warning #:postgresql-notification)
   (:export #:+postgresql-server-default-port+
 	   #:postgresql-condition
 	   #:postgresql-error
